use std::path::PathBuf;

use clap::{Args, Parser, Subcommand};

#[derive(Debug, Parser, PartialEq, Eq)]
#[command(version, about, author, long_about = None, args_override_self = true)]
pub struct CliArgs {
	#[command(subcommand)]
	pub commands: Commands,
}

#[derive(Debug, Subcommand, PartialEq, Eq)]
pub enum Commands {
	Init,
	Untag(UntagArgs),
	Tag {
		#[clap(flatten)]
		args: TagArgs,

		#[arg(short, long, requires = "file_tags", value_delimiter = ' ', num_args = 1..)]
		files: Vec<PathBuf>,

		#[arg(short('t'), long("tags"), requires = "files", value_delimiter = ' ', num_args = 1..)]
		file_tags: Vec<String>,
	},
	Tags(TagsArgs),
	Files(FilesArgs),
}

#[derive(Debug, Args, PartialEq, Eq)]
pub struct TagsArgs {
	pub query: Option<String>,

	#[command(subcommand)]
	pub commands: Option<TagsCommands>,
}

#[derive(Debug, Subcommand, PartialEq, Eq)]
pub enum TagsCommands {
	List,
	Add { add: Vec<String> },
	Remove { remove: Vec<String> },
	Rename { renames: Vec<String> },
}

#[derive(Debug, Args, PartialEq, Eq)]
pub struct TagArgs {
	// #[clap(conflicts_with = "files")]
	pub file: Option<PathBuf>,

	// #[clap(conflicts_with = "files")]
	pub tags: Vec<String>,
}

#[derive(Debug, Args, PartialEq, Eq)]
pub struct UntagArgs {
	pub file: PathBuf,

	pub tags: Option<Vec<String>>,
}

#[derive(Debug, Args, PartialEq, Eq)]
pub struct FilesArgs {
	pub query: Option<String>,

	#[arg(long, short, exclusive = true, default_value_t = false)]
	pub all: bool,
}
